diff --git a/modules/access/qtcapture.m b/modules/access/qtcapture.m
index 1a2ce64..83f2328 100644
--- a/modules/access/qtcapture.m
+++ b/modules/access/qtcapture.m
@@ -161,6 +161,181 @@ vlc_module_end ()
 
 @end
 
+@interface VLCDecompressedAudioOutput : QTCaptureDecompressedAudioOutput
+{
+    AudioBuffer *currentAudioBuffer;
+    mtime_t currentPts;
+	mtime_t mach_currentPts;
+    mtime_t previousPts;
+}
+- (id)init;
+- (void)outputAudioSampleBuffer:(QTSampleBuffer *)sampleBuffer fromConnection:(QTCaptureConnection *)connection;
+- (mtime_t)copyCurrentAudioBytesToBuffer:(void *)buffer;
+@end
+
+@implementation VLCDecompressedAudioOutput : QTCaptureDecompressedAudioOutput
+- (id)init
+{
+    if( self = [super init] )
+    {
+        currentAudioBuffer = nil;
+        currentPts = 0;
+        previousPts = 0;
+    }
+    return self;
+}
+- (void)dealloc
+{
+    @synchronized (self)
+    {
+        currentAudioBuffer = nil;
+    }
+    [super dealloc];
+}
+
+- (void)outputAudioSampleBuffer:(QTSampleBuffer *)sampleBuffer fromConnection:(QTCaptureConnection *)connection
+{
+	//NSLog(@"DEBUG outputAudioSampleBuffer:");
+	AudioBufferList *tempAudioBufferList;
+	NSMutableData *rawAudioData;
+	AudioSampleType *audioData;
+	//AudioStreamBasicDescription sampleBufferASBD;
+	int audioDataSize = 0;
+	int count = 0;
+	
+	@synchronized (self)
+    {
+        //audioBufferToRelease = malloc(sizeof(currentAudioBufferList));
+		//audioBufferToRelease = currentAudioBufferList;
+		tempAudioBufferList = [sampleBuffer audioBufferListWithOptions:QTSampleBufferAudioBufferListOptionAssure16ByteAlignment];
+		QTFormatDescription *formatDescription = [sampleBuffer formatDescription];
+		NSValue *sampleBufferASBDValue = [formatDescription attributeForKey:QTFormatDescriptionAudioStreamBasicDescriptionAttribute];
+		if (!sampleBufferASBDValue)
+		{
+			return;
+		}
+		
+		AudioStreamBasicDescription sampleBufferASBD = {0};
+		[sampleBufferASBDValue getValue:&sampleBufferASBD];
+		
+		audioData = (AudioSampleType *) calloc (sampleBufferASBD.mFramesPerPacket, sizeof (AudioSampleType));
+		
+		currentPts = (mtime_t)(1000000L / [sampleBuffer presentationTime].timeScale * [sampleBuffer presentationTime].timeValue);
+        //mach_currentPts = (mtime_t)mach_absolute_time();
+		// Try to use hosttime of the sample if available, as with video 
+        NSNumber *hosttime = (NSNumber *)[sampleBuffer attributeForKey:QTSampleBufferHostTimeAttribute];
+        if( hosttime ) currentPts = (mtime_t)AudioConvertHostTimeToNanos([hosttime unsignedLongLongValue])/1000;
+		
+		// Time debugging
+		//NSLog(@"currentPts %d",currentPts);
+		//NSLog(@"\noriginalPts: %@;\ntimeScale: %d;\ntimeValue: %d;\ntimeScale*timeValue: %lld;\nvlc_calculation: %d;\nhosttime: %i;\nmach_currentPts: %i",QTStringFromTime([sampleBuffer presentationTime]),[sampleBuffer presentationTime].timeScale,[sampleBuffer presentationTime].timeValue,([sampleBuffer presentationTime].timeScale * [sampleBuffer presentationTime].timeValue),(1000000L / [sampleBuffer presentationTime].timeScale * [sampleBuffer presentationTime].timeValue),[[sampleBuffer attributeForKey:QTSampleBufferHostTimeAttribute] intValue],mach_currentPts);
+		//NSLog(@"currentAudioBufferList: %d", tempAudioBufferList->mNumberBuffers);
+		NSLog(@"audio hosttime: %u",hosttime);
+		NSLog(@"audio currentPts: %u",currentPts);
+		
+		if (tempAudioBufferList->mNumberBuffers > 1)
+		{
+			for (count = 0; count < tempAudioBufferList->mNumberBuffers; count++) {
+				audioDataSize = (audioDataSize + tempAudioBufferList->mBuffers[count].mDataByteSize);
+				
+			}
+			//NSLog(@"audioDataSize: %d",audioDataSize);
+			rawAudioData = [[NSMutableData alloc] initWithLength:audioDataSize];
+		}
+		
+		for (count = 0; count < tempAudioBufferList->mNumberBuffers; count++) {
+			//NSLog(@"count: %d; channels: %d; size %d; data: %d",count,tempAudioBufferList->mBuffers[count].mNumberChannels,tempAudioBufferList->mBuffers[count].mDataByteSize,tempAudioBufferList->mBuffers[count].mData);
+			//[rawAudioData appendBytes:tempAudioBufferList->mBuffers[count].mData length:tempAudioBufferList->mBuffers[count].mDataByteSize];			
+			audioData = tempAudioBufferList->mBuffers[count].mData;
+		}
+		currentAudioBuffer = (AudioBuffer *)malloc(sizeof(AudioBuffer));
+		if (NULL == currentAudioBuffer) {
+			//NSLog (@"*** malloc failure for allocating audioBuffer memory");
+			[rawAudioData release];
+			return;
+		}
+		
+		//currentAudioBuffer->mNumberChannels = tempAudioBufferList->mBuffers[0].mNumberChannels;
+		currentAudioBuffer->mNumberChannels = 1;
+		currentAudioBuffer->mDataByteSize = sizeof(AudioSampleType);
+		currentAudioBuffer->mData = audioData;
+		//currentAudioBuffer->mData = tempAudioBufferList->mBuffers[0].mData;
+		//NSLog(@"currentAudioBuffer channels: %d; size: %d; data: %d",currentAudioBuffer->mNumberChannels,currentAudioBuffer->mDataByteSize,currentAudioBuffer->mData);
+    }
+	//NSLog(@"audio currentPts %d",currentPts);
+	tempAudioBufferList = nil;
+	[rawAudioData release];
+	//free(audioBufferToRelease);
+	//[audioBufferToRelease release];
+}
+
+- (mtime_t)copyCurrentAudioBytesToBuffer:(void *)buffer
+{
+	//NSLog(@"DEBUG copyCurrentAudioBytesToBuffer:");
+	AudioBuffer *audioBuffer;
+    mtime_t pts;
+	//void * data;
+	
+    if(!currentAudioBuffer|| currentPts == previousPts )
+	{
+		//NSLog(@"nothing there");
+		return 0;
+	}
+	
+    @synchronized (self)
+    {
+		audioBuffer = (AudioBuffer *)malloc(sizeof(currentAudioBuffer));
+		if(audioBuffer){
+			//NSLog(@"currentAudioBuffer channels: %d; size: %d; data: %d",currentAudioBuffer->mNumberChannels,currentAudioBuffer->mDataByteSize,currentAudioBuffer->mData);
+			pts = previousPts = currentPts;
+			memcpy( buffer, currentAudioBuffer->mData, currentAudioBuffer->mDataByteSize);
+		}
+        //audioBuffer = (AudioBufferList *)malloc(sizeof(AudioBufferList));
+		//audioBuffer = malloc(sizeof(currentAudioBufferList));
+		//NSLog(@"audiobuffer");
+		
+		/*
+		 if( !(currentAudioBufferList->mBuffers[0].mData) || (currentAudioBufferList->mNumberBuffers == 0) )
+		 {
+		 NSLog(@"No data yet");
+		 return 0;
+		 }
+		 */
+		
+		
+		/*
+		 NSMutableData *mergedAudioBufferData = [NSMutableData dataWithCapacity:sizeof(AudioBufferList)];
+		 int dataIndex;
+		 for (dataIndex = 0; dataIndex < currentAudioBufferList->mNumberBuffers; dataIndex++) {
+		 NSLog(@"merge");
+		 //NSLog(@"merging buffer: %@ into one buffer with data: ",currentAudioBufferList->mNumberBuffers, currentAudioBufferList->mBuffers[dataIndex].mData);
+		 [mergedAudioBufferData appendBytes:currentAudioBufferList->mBuffers[dataIndex].mData length:currentAudioBufferList->mBuffers[dataIndex].mDataByteSize];
+		 }
+		 
+		 NSLog(@"creating new AudioBuffer");
+		 audioBuffer->mNumberBuffers = 1;
+		 audioBuffer->mBuffers[0].mNumberChannels = currentAudioBufferList->mBuffers[0].mNumberChannels;
+		 audioBuffer->mBuffers[0].mDataByteSize = [mergedAudioBufferData length];
+		 audioBuffer->mBuffers[0].mData = [mergedAudioBufferData mutableBytes];
+		 */
+		//pts = previousPts = currentPts;
+		
+		//data = [mergedAudioBufferData mutableBytes];
+		//memcpy( buffer , data, 0);
+		//[mergedAudioBufferData release];
+		//[data release];
+    }
+	//free(audioBuffer);
+	free(audioBuffer);
+	
+	if(currentAudioBuffer->mData)
+		return currentPts;
+	else
+		return 0;
+}
+
+@end
+
 /*****************************************************************************
 * Struct
 *****************************************************************************/
@@ -168,9 +343,14 @@ vlc_module_end ()
 struct demux_sys_t {
     QTCaptureSession * session;
     QTCaptureDevice * device;
+	QTCaptureDevice * audiodevice;
     VLCDecompressedVideoOutput * output;
+	VLCDecompressedAudioOutput * audiooutput;
     int height, width;
+	int i_audio_max_frame_size;
+	int channels, i_sample_rate;
     es_out_id_t * p_es_video;
+	es_out_id_t * p_es_audio;
 };
 
 
@@ -208,6 +388,7 @@ static int Open( vlc_object_t *p_this )
     demux_t     *p_demux = (demux_t*)p_this;
     demux_sys_t *p_sys = NULL;
     es_format_t fmt;
+	es_format_t audiofmt;
     int i;
     int i_width;
     int i_height;
@@ -298,7 +479,8 @@ static int Open( vlc_object_t *p_this )
     }
 
     p_sys->output = [[VLCDecompressedVideoOutput alloc] init];
-
+	msg_Dbg ( p_demux, "initialized video output" );
+	
     /* Get the formats */
     NSArray *format_array = [p_sys->device formatDescriptions];
     QTFormatDescription* camera_format = NULL;
@@ -306,8 +488,8 @@ static int Open( vlc_object_t *p_this )
     {
         camera_format = [format_array objectAtIndex: k];
 
-        msg_Dbg(p_demux, "localized Format: %s", [[camera_format localizedFormatSummary] UTF8String] );
-        msg_Dbg(p_demux, "format description: %s", [[[camera_format formatDescriptionAttributes] description] UTF8String] );
+        msg_Dbg(p_demux, "Video localized format summary: %s", [[camera_format localizedFormatSummary] UTF8String] );
+        msg_Dbg(p_demux, "Video format description attributes: %s", [[[camera_format formatDescriptionAttributes] description] UTF8String] );
     }
     if( [format_array count] )
         camera_format = [format_array objectAtIndex: 0];
@@ -346,6 +528,91 @@ static int Open( vlc_object_t *p_this )
         [NSNumber numberWithInt: p_sys->width], kCVPixelBufferWidthKey,
         [NSNumber numberWithBool:YES], (id)kCVPixelBufferOpenGLCompatibilityKey,
         nil]];
+	
+	memset( &audiofmt, 0, sizeof( es_format_t ) ); 
+	
+	QTCaptureDeviceInput * audioInput = nil;
+    NSError *o_returnedAudioError;
+	
+	p_sys->audiodevice = [QTCaptureDevice defaultInputDeviceWithMediaType: QTMediaTypeSound];
+	
+	if (!p_sys->audiodevice)
+	{
+		p_sys->audiodevice = nil;
+		dialog_FatalWait( p_demux, _("No audio input device found"),
+						 _("Your Mac does not seem to be equipped with a suitable audio input device. "
+						   "Please check your connectors and drivers.") );
+        msg_Err( p_demux, "Can't find any audio device" );
+		goto error;
+	}
+	
+	if( ![p_sys->audiodevice open: &o_returnedAudioError] )
+    {
+        msg_Err( p_demux, "Unable to open the audio capture device (%ld)", [o_returnedAudioError code] );
+        goto error;
+    }
+	
+	if( [p_sys->audiodevice isInUseByAnotherApplication] == YES )
+    {
+        msg_Err( p_demux, "Default audio capture device is exclusively in use by another application" );
+        goto error;
+    }
+	
+	audioInput = [[QTCaptureDeviceInput alloc] initWithDevice: p_sys->audiodevice];
+	if( !audioInput )
+    {
+        msg_Err( p_demux, "Can't create a valid audio capture input facility" );
+        goto error;
+    } else {
+		msg_Dbg( p_demux, "Created valid audio capture input facility" );
+	}
+	
+	p_sys->audiooutput = [[VLCDecompressedAudioOutput alloc] init];
+	msg_Dbg ( p_demux, "initialized audio output" );
+	
+	NSArray *audioformat_array = [p_sys->audiodevice formatDescriptions];
+    QTFormatDescription* audio_format = NULL;
+    for( int k = 0; k < [audioformat_array count]; k++ )
+    {
+        audio_format = (QTFormatDescription *)[audioformat_array objectAtIndex: k];
+		
+		msg_Dbg( p_demux, "Audio localized format summary: %s", [[audio_format localizedFormatSummary] UTF8String]);
+		msg_Dbg( p_demux, "Audio format description attributes: %s",[[[audio_format formatDescriptionAttributes] description] UTF8String]);	
+		
+		AudioStreamBasicDescription audioDesc = {0};
+		NSValue *descDataValue =  [audio_format attributeForKey:QTFormatDescriptionAudioStreamBasicDescriptionAttribute];
+		[descDataValue getValue:&audioDesc];
+		char formatIDString[5];
+		UInt32 formatID = CFSwapInt32HostToBig (audioDesc.mFormatID);
+		bcopy (&formatID, formatIDString, 4);
+		formatIDString[4] = '\0';
+		
+		msg_Dbg ( p_demux, "Audio Sample Rate: %10.0f", audioDesc.mSampleRate);
+		msg_Dbg ( p_demux, "Audio Format ID: %10s", formatIDString);
+		msg_Dbg ( p_demux, "Audio Format Flags: %10X", audioDesc.mFormatFlags);
+		msg_Dbg ( p_demux, "Audio Bytes per Packet: %10d", audioDesc.mBytesPerPacket);
+		msg_Dbg ( p_demux, "Audio Frames per Packet: %10d", audioDesc.mFramesPerPacket);
+		msg_Dbg ( p_demux, "Audio Bytes per Frame: %10d", audioDesc.mBytesPerFrame);
+		msg_Dbg ( p_demux, "Audio Channels per Frame: %10d", audioDesc.mChannelsPerFrame);
+		msg_Dbg ( p_demux, "Audio Bits per Channel: %10d", audioDesc.mBitsPerChannel);
+		
+    }
+    if( [audioformat_array count] )
+        audio_format = [format_array objectAtIndex: 0];
+    else goto error;
+	
+	int audiocodec = VLC_FOURCC('a','r','a','w');
+	es_format_Init( &audiofmt, AUDIO_ES, audiocodec);
+	
+	/* FIXME: these values should probably be filled dynamically */
+	p_sys->i_audio_max_frame_size = 32 * 1024; /* Is this correct? */
+	p_sys->channels = 2;
+	p_sys->i_sample_rate = 44100;
+	audiofmt.audio.i_channels = 2;
+	audiofmt.audio.i_rate = 44100;
+	audiofmt.audio.i_bitspersample = 16;
+	audiofmt.audio.i_blockalign = audiofmt.audio.i_channels * audiofmt.audio.i_bitspersample / 8; /* Adapted from dc1394.c */
+	audiofmt.i_bitrate = audiofmt.audio.i_channels * audiofmt.audio.i_rate * audiofmt.audio.i_bitspersample; /* Adapted from dc1394.c */
 
     p_sys->session = [[QTCaptureSession alloc] init];
 
@@ -355,25 +622,42 @@ static int Open( vlc_object_t *p_this )
         msg_Err( p_demux, "default video capture device could not be added to capture session (%ld)", [o_returnedError code] );
         goto error;
     }
-
+	
+	ret = [p_sys->session addInput:audioInput error: &o_returnedError];
+    if( !ret )
+    {
+        msg_Err( p_demux, "the audio capture device could not be added to capture session (%ld)", [o_returnedError code] );
+        goto error;
+    }
+	
     ret = [p_sys->session addOutput:p_sys->output error: &o_returnedError];
     if( !ret )
     {
         msg_Err( p_demux, "output could not be added to capture session (%ld)", [o_returnedError code] );
         goto error;
     }
+	
+	ret = [p_sys->session addOutput:p_sys->audiooutput error: &o_returnedError];
+    if( !ret )
+    {
+        msg_Err( p_demux, "audio output could not be added to capture session (%ld)", [o_returnedError code] );
+        goto error;
+    }
 
     [p_sys->session startRunning];
 
     msg_Dbg( p_demux, "added new video es %4.4s %dx%d",
             (char*)&fmt.i_codec, fmt.video.i_width, fmt.video.i_height );
+	msg_Dbg( p_demux, "New audio es %d channels %dHz",
+            audiofmt.audio.i_channels, audiofmt.audio.i_rate );
 
     p_sys->p_es_video = es_out_Add( p_demux->out, &fmt );
+	p_sys->p_es_audio = es_out_Add( p_demux->out, &audiofmt );
 
     [input release];
     [pool release];
 
-    msg_Dbg( p_demux, "QTCapture: We have a video device ready!" );
+    msg_Dbg( p_demux, "QTCapture: We have a video and audio device ready!" );
 
     return VLC_SUCCESS;
 error:
@@ -404,6 +688,7 @@ static void Close( vlc_object_t *p_this )
         // work on main thread. And this will create a dead lock.
         [p_sys->session performSelectorOnMainThread:@selector(stopRunning) withObject:nil waitUntilDone:NO];
         [p_sys->output performSelectorOnMainThread:@selector(release) withObject:nil waitUntilDone:NO];
+		[p_sys->audiooutput performSelectorOnMainThread:@selector(release) withObject:nil waitUntilDone:NO];
         [p_sys->session performSelectorOnMainThread:@selector(release) withObject:nil waitUntilDone:NO];
     }
     free( p_sys );
@@ -419,6 +704,7 @@ static int Demux( demux_t *p_demux )
 {
     demux_sys_t *p_sys = p_demux->p_sys;
     block_t *p_block;
+	block_t *p_block_audio;
 
     p_block = block_New( p_demux, p_sys->width *
                             p_sys->height * 2 /* FIXME */ );
@@ -427,6 +713,14 @@ static int Demux( demux_t *p_demux )
         msg_Err( p_demux, "cannot get block" );
         return 0;
     }
+	
+	p_block_audio = block_New( p_demux, p_sys->i_audio_max_frame_size );
+	
+	if( !p_block_audio )
+    {
+        msg_Err( p_demux, "cannot get audio block" );
+        return 0;
+    }
 
     NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
 
@@ -443,11 +737,30 @@ static int Demux( demux_t *p_demux )
         msleep( 10000 );
         return 1;
     }
+	
+	@synchronized (p_sys->audiooutput)
+    {
+		p_block_audio->i_pts = [p_sys->audiooutput copyCurrentAudioBytesToBuffer: p_block_audio->p_buffer];
+    }
+	
+	if( !p_block_audio->i_pts )
+    {
+        // Nothing to transfer yet, just forget
+        block_Release( p_block_audio );
+        [pool release];
+        msleep( 10000 );
+        return 1;
+    }
 
     es_out_Control( p_demux->out, ES_OUT_SET_PCR, p_block->i_pts );
     es_out_Send( p_demux->out, p_sys->p_es_video, p_block );
+	
+	es_out_Control( p_demux->out, ES_OUT_SET_PCR, p_block_audio->i_pts );
+    es_out_Send( p_demux->out, p_sys->p_es_audio, p_block_audio );
 
     [pool release];
+	
+	
     return 1;
 }
 
